# Description: This file contains all the enums used in the project

from enum import Enum


# Create enum for print type
class OutType(str, Enum):
    pretty = "pretty"
    machine = "machine"
    table = "table"
    json = "json"


class NextOutType(str, Enum):
    pretty = "pretty"
    machine = "machine"
    table = "table"
    json = "json"
    waybar = "waybar"


# Create enum for time format
class TimeFormat(str, Enum):
    twelve = "12"
    twenty_four = "24"


# Create enum for prayer
class Prayer(str, Enum):
    fajr = "Fajr"
    sunrise = "Sunrise"
    dhuhr = "Dhuhr"
    asr = "Asr"
    sunset = "Sunset"
    maghrib = "Maghrib"
    isha = "Isha"
    imsak = "Imsak"
    midnight = "Midnight"
    firstthird = "Firstthird"
    lastthird = "Lastthird"



# Description: Utility functions
from myprayer.cli.constants import TIMEDELTA
from myprayer.cli.enums import NextOutType, OutType


def get_key(my_dict, target_value):
    for key, value in my_dict.items():
        if value == target_value:
            return key


def format_time_left(time_delta, out_type: OutType | str) -> str:
    seconds = time_delta.seconds
    if isinstance(out_type, OutType) or isinstance(out_type, NextOutType):
        format = TIMEDELTA[out_type]
    else:
        format = out_type

    hours = seconds // 3600
    minutes = (seconds % 3600) // 60
    return format.format(hours=hours, minutes=minutes)



from datetime import datetime, timedelta

import tzlocal
from adhanpy.calculation import CalculationMethod
from adhanpy.PrayerTimes import PrayerTimes

tz = tzlocal.get_localzone()


class Prayer:
    def __init__(self, name: str, time: datetime) -> None:
        self.name = name
        self.time = time

    def has_passed(self) -> bool:
        return datetime.now(tz) > self.time

    def time_left(self) -> timedelta:
        return self.time - datetime.now(tz)

    def __str__(self) -> str:
        return f"{self.name}: {self.time.strftime('%H:%M')}"


class Day:
    """Represents a single day with prayer times.

    Attributes:
        day (int): The day number in the month
        month (int): The month number 1-12
        year (int): The year
        data (dict): The prayer time data for this day
        prayers (list[Prayer]): List of Prayer objects
        skip (list[str]): Prayer names to skip

    Methods:
        get_next_prayer(): Returns the next prayer that has not passed yet
        get_prayer(name): Returns the Prayer object with the given name
        has_passed(): Checks if the last prayer of the day has passed

    Raises:
        ValueError: If day is not 1-31 or month is not 1-12

    Examples:
        >>> data = Client().get_day(15, 1, 2022)
        >>> print(day.prayers[0].name)
        Fajr

        >>> day.get_prayer("fajr").__str__()
        Fajr: 05:20
    """

    latitude: float
    longitude: float
    method: CalculationMethod
    date: datetime
    prayers: list[Prayer]
    skip: list[str]

    def __init__(
        self,
        latitude: float = 30,
        longitude: float = 31,
        method: CalculationMethod = CalculationMethod.EGYPTIAN,
        date: datetime = datetime.now(tz),
        skip: list[str] = [],
    ):
        self.latitude = latitude
        self.longitude = longitude
        self.method = method
        self.skip = [x.lower() for x in skip]

        self.date: datetime = date
        prayers = self.__get_prayer_times(latitude, longitude, date, method)

        for prayer in prayers:
            if prayer.name.lower() in self.skip:
                prayers.remove(prayer)

        self.prayers: list[Prayer] = prayers

    @staticmethod
    def __get_prayer_times(
        latitude: float,
        longitude: float,
        date: datetime,
        method: CalculationMethod = CalculationMethod.EGYPTIAN,
    ):
        prayer_times = PrayerTimes(
            (latitude, longitude),
            date,
            method,
            time_zone=tz,
        )

        return [
            Prayer("Fajr", prayer_times.fajr),
            Prayer("Sunrise", prayer_times.sunrise),
            Prayer("Dhuhr", prayer_times.dhuhr),
            Prayer("Asr", prayer_times.asr),
            Prayer("Maghrib", prayer_times.maghrib),
            Prayer("Isha", prayer_times.isha),
        ]

    def next(self) -> None:
        self.__init__(
            self.latitude,
            self.longitude,
            self.method,
            self.date + timedelta(days=1),
            self.skip,
        )

    def get_next_prayer(self) -> Prayer | None:
        for prayer in self.prayers:
            if not prayer.has_passed():
                return prayer

    def get_prayer(self, name: str) -> Prayer | None:
        for prayer in self.prayers:
            if prayer.name == name:
                return prayer

    def has_passed(self) -> bool:
        return self.prayers[-1].has_passed()



#!/usr/bin/env python

import json
from datetime import datetime
from importlib.metadata import version as get_version
from typing import Optional

import inquirer
import typer
import tzlocal
from adhanpy.calculation import CalculationMethod
from geopy import Nominatim
from rich import print as rprint
from rich.prompt import FloatPrompt, Prompt
from rich.table import Table

from myprayer.cli import utils
from myprayer.cli.config import Config, Coordinates
from myprayer.cli.constants import (
    APP_NAME,
    CONFIG_FILE,
    LOCATION_TYPES,
    PRAYERS,
    TIME_FORMATS,
)
from myprayer.cli.day import Day
from myprayer.cli.enums import NextOutType, OutType, TimeFormat
from myprayer.cli.output import DayOutput
from myprayer.cli.utils import format_time_left

app = typer.Typer(name=APP_NAME, pretty_exceptions_enable=False, help="MyPrayer CLI.")


# TODO: Emphasize next prayer in waybar output <span font_weight="bold">...</span>

# Load config
CONFIG: Config = Config(CONFIG_FILE)
SKIP = [prayer for prayer in PRAYERS if prayer not in CONFIG.prayers]
# get current timezone
tz = tzlocal.get_localzone()


def get_coordinates(address: str):
    nom = Nominatim(user_agent="myprayer")
    location = nom.geocode(address)

    return location.latitude, location.longitude  # type: ignore


@app.command(name="list", help="List prayer times.")
def list_prayers(
    city: str = typer.Option(
        None,
        "--city",
        "-c",
        help="City name.",
        show_default=False,
    ),
    country: str = typer.Option(
        None,
        "--country",
        "-C",
        help="Country name.",
        show_default=False,
    ),
    address: str = typer.Option(
        None,
        "--address",
        "-a",
        help="Address.",
        show_default=False,
    ),
    latitude: float = typer.Option(
        CONFIG.location.latitude if CONFIG.location.latitude else None,
        "--latitude",
        "-lat",
        help="Latitude.",
        show_default=True,
    ),
    longitude: float = typer.Option(
        CONFIG.location.longitude if CONFIG.location.longitude else None,
        "--longitude",
        "-lon",
        help="Longitude.",
        show_default=True,
    ),
    date_iso: datetime = typer.Option(
        None,
        "--date",
        "-d",
        help="Date (YYYY-MM-DD) ISO 8601",
        show_default="Current date",  # type: ignore
    ),
    method: int = typer.Option(
        CONFIG.method,
        "--method",
        "-M",
        help="Calculation method.",
        min=0,
        max=CalculationMethod.__len__() - 1,
    ),
    time_format: TimeFormat = typer.Option(
        CONFIG.time_format,
        "--time-format",
        "-t",
        help="Time format.",
        show_default=f"{TimeFormat(CONFIG.time_format).value}",  # type: ignore
    ),
    custom_time_format: str = typer.Option(
        CONFIG.custom_time_format,
        "--custom-time-format",
        "-T",
        help="Custom time format.",
        show_default=True,  # type: ignore
    ),
    out_type: OutType = typer.Option(
        CONFIG.out_type,
        "--output",
        "-o",
        help="Output type.",
        show_default=f"{OutType(CONFIG.out_type).value}",  # type: ignore
    ),
    next: bool = typer.Option(
        CONFIG.next,
        "--next",
        "-n",
        help="Show next prayer, has no effect if day, month, or year are given.",
    ),
):
    if CONFIG.is_error:
        typer.echo(message=f"[ERROR] {CONFIG.error}", err=True)
        exit(1)

    # client = get_client(city, country, address, latitude, longitude, method, force)

    if city and country:
        latitude, longitude = get_coordinates(f"{city}, {country}")
    elif address:
        latitude, longitude = get_coordinates(address)
    elif latitude and longitude:
        pass
    else:
        latitude, longitude = CONFIG.location.latitude, CONFIG.location.longitude

    date = (
        date_iso.replace(tzinfo=tz) if date_iso else datetime.today().replace(tzinfo=tz)
    )

    day_data = Day(latitude, longitude, CalculationMethod(method), date, SKIP)

    if date.date() == datetime.now(tz).date():
        if day_data.has_passed():
            day_data.next()
    else:
        next = False

    used_time_format = (
        custom_time_format if custom_time_format else TIME_FORMATS[time_format]
    )

    output = DayOutput(day_data, used_time_format, next)

    if out_type == OutType.table:
        rprint(output.table())
    elif out_type == OutType.pretty:
        rprint(output.pretty())
    elif out_type == OutType.machine:
        print(output.machine())
    elif out_type == OutType.json:
        print(json.dumps(output.json(), indent=4))


@app.command(name="next", help="Show next prayer.")
def next(
    city: str = typer.Option(
        None,
        "--city",
        "-c",
        help="City name.",
        show_default=False,
    ),
    country: str = typer.Option(
        None,
        "--country",
        "-C",
        help="Country name.",
        show_default=False,
    ),
    address: str = typer.Option(
        None,
        "--address",
        "-a",
        help="Address.",
        show_default=False,
    ),
    latitude: float = typer.Option(
        CONFIG.location.latitude if CONFIG.location.latitude else None,
        "--latitude",
        "-lat",
        help="Latitude.",
        show_default=True,
    ),
    longitude: float = typer.Option(
        CONFIG.location.longitude if CONFIG.location.longitude else None,
        "--longitude",
        "-lon",
        help="Longitude.",
        show_default=True,
    ),
    method: int = typer.Option(
        CONFIG.method,
        "--method",
        "-M",
        help="Calculation method.",
        min=0,
        max=CalculationMethod.__len__() - 1,
    ),
    out_type: NextOutType = typer.Option(
        CONFIG.out_type,
        "--output",
        "-o",
        help="Output type.",
        show_default=f"{NextOutType(CONFIG.out_type).value}",  # type: ignore
    ),
):
    if CONFIG.is_error:
        typer.echo(message=f"[ERROR] {CONFIG.error}", err=True)
        exit(1)

    if city and country:
        latitude, longitude = get_coordinates(f"{city}, {country}")
    elif address:
        latitude, longitude = get_coordinates(address)
    elif latitude and longitude:
        pass
    else:
        latitude, longitude = CONFIG.location.latitude, CONFIG.location.longitude

    today = datetime.now(tz)
    # day_data = client.get_day(day, month, year)
    day_data = Day(latitude, longitude, CalculationMethod(method), today, SKIP)

    if day_data.has_passed():
        day_data.next()

    next_prayer = day_data.get_next_prayer()

    if next_prayer is not None:
        time_left = format_time_left(next_prayer.time_left(), out_type)  # type: ignore
        if out_type == OutType.table:
            table = Table(show_header=True, header_style="bold magenta")
            table.add_column("Prayer")
            table.add_column("Time Left")
            table.add_row(next_prayer.name, time_left, style="bold")
            rprint(table)
        elif out_type == OutType.pretty:
            rprint(f"[bold cyan]{next_prayer.name}:[/bold cyan] {time_left}")
        elif out_type == OutType.machine:
            print(f"{next_prayer.name},{time_left}")
        elif out_type == OutType.json:
            out_json = {
                "next": next_prayer.name,
                "time_left": time_left,
            }
            print(json.dumps(out_json, indent=4))
        elif out_type == NextOutType.waybar:
            tooltip_date = day_data.date.strftime("%A, %B %d")
            tooltip_data = "\n".join(
                [
                    f"{prayer.name}: {prayer.time.strftime(TIME_FORMATS[CONFIG.time_format])}"
                    for prayer in day_data.prayers
                ]
            )
            tooltip = f"{tooltip_date}\n\n{tooltip_data}"

            out_json = {
                "text": f"{time_left}",
                "tooltip": tooltip,
                "class": next_prayer.name.lower(),
                "alt": f"{next_prayer.name}: {time_left}",
            }

            print(json.dumps(out_json, indent=4))


@app.command(name="config", help="Configure myprayer.")
def config():

    # Prompt for city
    loc_type_question = [
        inquirer.List(
            "type",
            message="Select a location type:",
            choices=LOCATION_TYPES,
            default=type(CONFIG.location).__name__,  # type: ignore
        ),
    ]
    loc_type_choice = inquirer.prompt(loc_type_question)
    loc_type: str = loc_type_choice["type"]  # type: ignore

    latitude = 30
    longitude = 31
    if loc_type == "City":
        city: str = Prompt.ask(
            "City",
        )
        country: str = Prompt.ask(
            "Country",
        )
        state: str = Prompt.ask(
            "State",
            default=None,  # type: ignore
        )
        location = f"{city}, {country}, {state}" if state else f"{city}, {country}"
        latitude, longitude = get_coordinates(location)

    elif loc_type == "Coordinates":
        latitude: float = FloatPrompt.ask(
            "Latitude",
            default=(
                CONFIG.location.latitude if CONFIG.location.latitude else None
            ),  # type: ignore
        )
        longitude: float = FloatPrompt.ask(
            "Longitude",
            default=(
                CONFIG.location.longitude if CONFIG.location.longitude else None
            ),  # type: ignore
        )

    elif loc_type == "Address":
        address: str = Prompt.ask(
            "Address",
        )
        latitude, longitude = get_coordinates(address)

    # Prompt for calculation method
    method_question = [
        inquirer.List(
            "method",
            message="Select a calculation method:",
            choices=CalculationMethod.__members__.keys(),
            default=utils.get_key(
                CalculationMethod.__members__, CalculationMethod(CONFIG.method)
            ),  # type: ignore
        ),
    ]

    method_choice = inquirer.prompt(method_question)

    if method_choice is None:
        raise typer.Abort()

    method: int = CalculationMethod[method_choice["method"]].value

    # Prompt for time format
    custom_time_format: Optional[str] = None
    time_format: str = TimeFormat(CONFIG.time_format).value

    is_custom_time_format_question = [
        inquirer.Confirm(
            "is_custom_time_format",
            message="Use custom time format?",
            default=CONFIG.custom_time_format is not None,
        )
    ]

    is_custom_time_format = inquirer.prompt(is_custom_time_format_question)

    if is_custom_time_format is None:
        raise typer.Abort()

    if is_custom_time_format["is_custom_time_format"]:
        custom_time_format = Prompt.ask(
            "Time format",
            default=(
                CONFIG.custom_time_format if CONFIG.custom_time_format else "%I:%M %p"
            ),
        )

    else:
        time_format: str = Prompt.ask(
            "Time format",
            choices=[
                TimeFormat.twelve,
                TimeFormat.twenty_four,
            ],
            default=TimeFormat.twelve.value,
        )

    # Prompt for print type
    print_type: str = Prompt.ask(
        "Output type",
        choices=[OutType.pretty, OutType.machine, OutType.table],
        default=OutType.table.value,
    )

    # Prompt for prayers to show

    prayers_question = [
        inquirer.Checkbox(
            "prayers",
            message="Select prayers to show:",
            choices=PRAYERS,
            default=CONFIG.prayers,
        ),
    ]
    prayers_choice = inquirer.prompt(prayers_question)
    prayers: list[str] = prayers_choice["prayers"]  # type: ignore

    # Prompt for next prayer option
    next = typer.confirm("Show next prayer?", default=True)

    CONFIG.update(
        location=Coordinates(latitude, longitude),
        custom_time_format=custom_time_format,
        time_format=TimeFormat(time_format),
        out_type=OutType(print_type),
        method=method,
        next=next,
        prayers=prayers,
    )
    CONFIG.save(CONFIG_FILE)

    rprint(f"[green]✔[/green] Configuration saved to {CONFIG_FILE}.")


def version_callback(value: bool):
    if value:
        print(f"{APP_NAME} {get_version(APP_NAME)}")
        raise typer.Exit()


def method_callback(value: bool):
    if value:
        for method in CalculationMethod:
            print(f"{method.value}: {method.name}")
        raise typer.Exit()


@app.callback()
def version(
    version: bool = typer.Option(
        None,
        "--version",
        "-v",
        callback=version_callback,
        is_eager=True,
        help="Print the version and exit.",
    ),
    method: bool = typer.Option(
        None,
        "--method",
        "-M",
        callback=method_callback,
        help="Print the calculation methods and exit.",
    ),
):
    pass


if __name__ == "__main__":
    app()



from rich import print as rprint
from rich.table import Table

from myprayer.cli.constants import TIME_FORMATS
from myprayer.cli.day import Day
from myprayer.cli.enums import OutType, TimeFormat
from myprayer.cli.utils import format_time_left


class DayOutput:
    day: Day
    time_format: str
    show_next: bool

    def __init__(self, day: Day, time_format: str, show_next: bool = False) -> None:
        self.day = day
        self.show_next = show_next
        self.time_format = time_format

    def table(self) -> Table:
        # table = Table(show_header=True, header_style="bold", border_style="magenta")
        table = Table(show_header=True, header_style="bold magenta")
        table.add_column("Prayer")
        table.add_column("Time")

        # print the date
        rprint(f"[bold]{self.day.date.strftime('%a %B %d %Y')}[/bold]")

        for prayer in self.day.prayers:
            is_next = self.day.get_next_prayer() == prayer
            if self.show_next and is_next:
                time_until = format_time_left(prayer.time_left(), OutType.table)
                table.add_row(
                    f"{prayer.name} ({time_until})",
                    prayer.time.strftime(self.time_format),
                    style="bold cyan",
                )
            else:
                table.add_row(
                    prayer.name,
                    prayer.time.strftime(self.time_format),
                    style="bold",
                )

        return table

    def pretty(self) -> str:
        output = ""

        # print the date
        output += f"[bold]{self.day.date.strftime('%a %B %d %Y')}[/bold]\n\n"

        for i, prayer in enumerate(self.day.prayers):
            formatted_time = prayer.time.strftime(self.time_format)
            is_next = self.day.get_next_prayer() == prayer
            if self.show_next and is_next:
                if is_next:
                    time_left = format_time_left(prayer.time_left(), OutType.pretty)
                    output += f"[bold cyan]{prayer.name}:[/bold cyan] {formatted_time} ({time_left})"
            else:
                output += f"[bold]{prayer.name}: {formatted_time}[/bold]"
            if i != len(self.day.prayers) - 1:
                output += "\n"

        return output

    def machine(self) -> str:
        output = ""
        for i, prayer in enumerate(self.day.prayers):
            formatted_time = prayer.time.strftime(self.time_format).replace(" ", "")

            prayer_output = f"{prayer.name},{formatted_time},{prayer.time.strftime('%Y-%m-%dT%H:%M:%S%z')}"

            if self.show_next:
                is_next = self.day.get_next_prayer() == prayer
                if is_next:
                    time_left = format_time_left(prayer.time_left(), OutType.machine)
                    prayer_output += f",{time_left}"

            output += prayer_output
            if i != len(self.day.prayers) - 1:
                output += "\n"
        return output

    def json(self) -> dict:
        out_json = {
            "date": self.day.date.strftime("%Y-%m-%d"),
            "timings": {
                prayer.name: prayer.time.strftime(self.time_format)
                for prayer in self.day.prayers
            },
        }
        next_prayer = self.day.get_next_prayer()
        if self.show_next and next_prayer is not None:
            time_left = format_time_left(next_prayer.time_left(), OutType.json)
            out_json["next"] = next_prayer.name
            out_json["time_left"] = time_left

        return out_json
